{******************************************************************************

Rotated label / expression for Quickreport 2 - Version 1.00
***********************************************************
Freeware (C) Copyright 1997 THSD Software, Timo Hartmann
E-mail: thsd@thsd.de  -  WWW: http://www.thsd.de

This unit is distributed as freeware. You can use it in your applications
free of charge, and you may modify it for your own use, but you are not allowed
to distribute modified versions of this source code.

TQRRotatedLabel and TQRRotatedExpr implement a Quickreport label/expression
which can display rotated text in 1/10 degree steps (i.e. the degree property
can be between 0 and 3600). If degree is set to 0 then the components behave
exactly like the original Quickreport components, so you can use the new
rotated components as a full replacement for TQRLabel and TQRExpr.
Note that currently "Autosize" is not supported if degree <> 0.

There is no TQRRotatedDBText included because you can easily use TQRRotatedExpr
to display datafields.

Installation:
**************
Install this unit into Delphi's component palette as usual.

Advertisement:
***************
Do you want your users to be able to modify their reports on their own? Check
out QRDesign, our end user report designer for both Quickreport 1 and 2.
QRDesign lets your end user create, modify, load and save his reports easily.
Additionally QRDesign adds many features to Quickreport and of course has lots
more features than the Quickreport runtime editor. A feature list, screenshots
and a demo version can be found on our website at http://www.thsd.de .


History:
********
June 21, 1997: Version 0.01 - first release
June 24, 1997: Version 0.02 - bugfix
July 07, 1997: Version 1.00 - "alignment" supported, added TQRRotatedExpr


If you found any bugs, have a question or suggestion then please send an email
to thsd@thsd.de !

******************************************************************************}

unit qr2rlabl;

interface

uses
  Classes, QuickRpt, QRCtrls;

type
  TQRRotatedLabel = class(TQRLabel)
  private
    FDegree : Word;
  protected
    procedure SetDegree(Value: Word);
  public
    procedure Paint; override;
    procedure Print(OfsX, OfsY: Integer); override;
  published
    property Degree : Word Read FDegree Write SetDegree;
  end;

  TQRRotatedExpr = class(TQRExpr)
  private
    FDegree : Word;
  protected
    procedure SetDegree(Value: Word);
  public
    procedure Paint; override;
    procedure Print(OfsX, OfsY: Integer); override;
  published
    property Degree : Word Read FDegree Write SetDegree;
  end;

procedure Register;

implementation

uses
  WinTypes, WinProcs, Graphics, Forms, QRPrntr, SysUtils;

{--------------------------------------------------------------------------------}
{--------------------------------------------------------------------------------}

function Min_Int(a, b: integer): integer;
begin
  if a < b then
    Result := a
  else
    Result := b;
end;

function Max_Int(a, b: integer): integer;
begin
  if a > b then
    Result := a
  else
    Result := b;
end;

procedure CalcRotatedPosition(TW, TH: Integer; Degree: Word; var AX, AY: Integer;
                               Width, Height: Integer; Alignment: TAlignment);
var
  BX, BY,
  DX, DY: Integer;
begin
  BX:=Round(TW*cos((Degree/10)*Pi/180));
  BY:=-Round(TW*sin((Degree/10)*Pi/180));
  DX:=Round(TH*sin(Degree/10*Pi/180));
  DY:=Round(TH*cos(Degree/10*Pi/180));
  AX:=-Min_Int(0,Min_Int(BX,Min_Int(BX+DX,DX)));
  AY:=-Min_Int(0,Min_Int(BY,Min_Int(BY+DY,DY)));
  If Alignment=taLeftJustify Then
    begin
      If (Degree>1350) and (Degree<=1800) Then AX:=Width-DX Else
      If (Degree>1800) and (Degree<=2250) Then AX:=Width;
      If (Degree>450) and (Degree<=900) Then AY:=Height-DY Else
      If (Degree>900) and (Degree<=1350) Then AY:=Height;
    end
  Else
  If Alignment=taRightJustify Then
    begin
      If (Degree>0) and (Degree<=450) Then AX:=-Min_Int(0,Min_Int(BX,Min_Int(BX+DX,DX)))+(Width-(BX+DX)) Else
      If (Degree>3150) and (Degree<=3600) Then AX:=-Min_Int(0,Min_Int(BX,Min_Int(BX+DX,DX)))+(Width-BX+DX);
      If (Degree>2250) and (Degree<=2700) Then AY:=-Min_Int(0,Min_Int(BY,Min_Int(BY+DY,DY)))+(Height-BY+DY) Else
      If (Degree>=2700) and (Degree<=3150) Then AY:=-Min_Int(0,Min_Int(BY,Min_Int(BY+DY,DY)))+(Height-BY-DY);
    end
  Else
    begin {taCenter}
      If (Degree>0) and (Degree<=450) Then AX:=-Min_Int(0,Min_Int(BX,Min_Int(BX+DX,DX)))+((Width-(BX+DX)) DIV 2) Else
      If (Degree>1350) and (Degree<=1800) Then AX:=(-Min_Int(0,Min_Int(BX,Min_Int(BX+DX,DX)))+(Width-DX)) DIV 2 ELse
      If (Degree>1800) and (Degree<=2250) Then AX:=(Width-Min_Int(0,Min_Int(BX,Min_Int(BX+DX,DX)))) DIV 2 Else
      If (Degree>2700) and (Degree<=3600) Then AX:=(-2*Min_Int(0,Min_Int(BX,Min_Int(BX+DX,DX)))+(Width-BX+DX)) DIV 2;
      If (Degree>450) and (Degree<=900) Then AY:=(-Min_Int(0,Min_Int(BY,Min_Int(BY+DY,DY)))+(Height-DY)) DIV 2 Else
      If (Degree>900) and (Degree<=1350) Then AY:=(-Min_Int(0,Min_Int(BY,Min_Int(BY+DY,DY)))+Height) DIV 2 Else
      If (Degree>2250) and (Degree<=2700) Then AY:=(-2*Min_Int(0,Min_Int(BY,Min_Int(BY+DY,DY)))+(Height-BY+DY)) DIV 2 Else
      If (Degree>=2700) and (Degree<=3150) Then AY:=(-2*Min_Int(0,Min_Int(BY,Min_Int(BY+DY,DY)))+(Height-BY-DY)) DIV 2;
    end;
end;

procedure DrawFrame(C: TQRPrintable; aCanvas: TCanvas; Rect: TRect);
var
  FWX, FWY: Integer;
begin
  dec(Rect.Bottom);
  dec(Rect.Right);
  FWX := round((Screen.PixelsPerInch / 254) / 72 * 254 * C.Frame.Width);
  FWY := round((Screen.PixelsPerInch / 254) / 72 * 254 * C.Frame.Width);
  aCanvas.Brush.Style := bsSolid;
  aCanvas.Brush.Color := C.Frame.Color;
  aCanvas.Pen.Style := psSolid;
  aCanvas.Pen.Color := C.Frame.Color;
  with aCanvas do
  begin
    if C.Frame.DrawTop then
      Rectangle(Rect.Left, Rect.Top, Rect.Right, Rect.Top + FWY);
    if C.Frame.DrawBottom then
      Rectangle(Rect.Left, Rect.Bottom, Rect.Right, Rect.Bottom - FWY);
    if C.Frame.DrawLeft then
      Rectangle(Rect.Left, Rect.Top, Rect.Left + FWX, Rect.Bottom);
    if C.Frame.DrawRight then
      Rectangle(Rect.Right - FWX, Rect.Top, Rect.Right, Rect.Bottom);
  end;
  aCanvas.Brush.Style := bsClear;
end;

procedure PrintFrame(C: TQRPrintable; aCanvas: TCanvas; Rect: TRect; WX, WY: Extended);
var
  FWX, FWY: Integer;
begin
  dec(Rect.Bottom);
  dec(Rect.Right);
  FWX := round(WX / 72 * 254 * C.Frame.Width);
  FWY := round(WY / 72 * 254 * C.Frame.Width);
  aCanvas.Brush.Style := bsSolid;
  aCanvas.Brush.Color := C.Frame.Color;
  aCanvas.Pen.Style := psSolid;
  aCanvas.Pen.Color := C.Frame.Color;
  with aCanvas do
  begin
    if C.Frame.DrawTop then
      Rectangle(Rect.Left, Rect.Top, Rect.Right, Rect.Top + FWY);
    if C.Frame.DrawBottom then
      Rectangle(Rect.Left, Rect.Bottom, Rect.Right, Rect.Bottom - FWY);
    if C.Frame.DrawLeft then
      Rectangle(Rect.Left, Rect.Top, Rect.Left + FWX, Rect.Bottom);
    if C.Frame.DrawRight then
      Rectangle(Rect.Right - FWX, Rect.Top, Rect.Right, Rect.Bottom);
  end;
  aCanvas.Brush.Style := bsClear;
end;

{--------------------------------------------------------------------------------}
{--------------------------------------------------------------------------------}

procedure TQRRotatedLabel.SetDegree(Value: Word);
begin
  If Value<>FDegree Then
    begin
      If Value<>0 Then AutoSize:=False;
      While Value>3600 Do Value:=Value-3600;
      FDegree:=Value;
      Invalidate;
    end;
end;

procedure TQRRotatedLabel.Paint;
var
  LogRec: TLOGFONT;
  Cap: String;
  AX, AY: Integer;
  OldFont: HFont;
begin
  If Degree=0 Then Inherited Paint
  else
    begin
      if not Transparent then
        begin
          Canvas.Brush.Color := Color;
          Canvas.Brush.Style := bsSolid;
          Canvas.Fillrect(Rect(0,0,Width,Height));
        end;
      Canvas.Font.Assign(Font);
      GetObject(Canvas.Font.Handle,SizeOf(LogRec),Addr(LogRec));
      LogRec.lfEscapement := FDegree;
      OldFont := SelectObject(Canvas.Handle,CreateFontIndirect(LogRec));
      Cap:=Caption;
      CalcRotatedPosition(Canvas.TextWidth(Cap),Canvas.TextHeight(Cap),Degree,AX,AY,Width,Height,Alignment);
      ExtTextOut(Canvas.Handle, AX, AY, 0, NIL, @Cap[1], Length(Cap), nil);
      DeleteObject(SelectObject(Canvas.Handle,OldFont));
      PaintCorners;
    end;
  DrawFrame(Self,Canvas,GetClientRect);
end;


procedure TQRRotatedLabel.Print(OfsX, OfsY: Integer);
var
  LogRec: TLOGFONT;
  Cap: String;
  AX, AY: Integer;
  R: TRect;
  OldFont: HFont;
begin
  If Degree=0 Then
    begin
      Inherited Print(OfsX, OfsY);
      exit;
    end;
  With ParentReport.QRPrinter Do
    begin
      if not Transparent then
        begin
          Canvas.Brush.Color := Color;
          Canvas.Brush.Style := bsSolid;
          Canvas.Fillrect(Rect(XPos(OfsX + Size.Left),YPos(OfsY + Size.Top),
                               XPos(OfsX + Size.Left+Size.Width),YPos(OfsY + Size.Top+Size.Height)));
        end;
      Canvas.Font.Assign(Font);
      GetObject(Canvas.Font.Handle,SizeOf(LogRec),Addr(LogRec));
      LogRec.lfEscapement := FDegree;
      OldFont := SelectObject(Canvas.Handle,CreateFontIndirect(LogRec));
      Cap:=Caption;
      CalcRotatedPosition(Canvas.TextWidth(Cap),Canvas.TextHeight(Cap),Degree,AX,AY,Width,Height,Alignment);
      R:= Rect( XPos(Size.Left), YPos(Size.Top),
                XPos(Size.Left+Size.Width),
                YPos(Size.Top+Size.Height));
      ExtTextOut(Canvas.Handle, XPos(OfsX + Size.Left + AX/XFactor), YPos(OfsY + Size.Top + AY/YFactor),
                 ETO_CLIPPED, @R, @Cap[1], Length(Cap), nil);
      DeleteObject(SelectObject(Canvas.Handle,OldFont));

      if ParentReport.FinalPass and Frame.AnyFrame then
        PrintFrame(Self,ParentReport.QRPrinter.Canvas,
                   Rect(ParentReport.QRPrinter.XPos(OfsX + Size.Left),
                        ParentReport.QRPrinter.YPos(OfsY + Size.Top),
                        ParentReport.QRPrinter.XPos(OfsX + Size.Left + Size.Width),
                        ParentReport.QRPrinter.YPos(OfsY + Size.Top + Size.Height)),
                   ParentReport.QRPrinter.XFactor,
                   ParentReport.QRPrinter.YFactor);
    end;
end;

{--------------------------------------------------------------------------------}
{--------------------------------------------------------------------------------}

procedure TQRRotatedExpr.Paint;
var
  LogRec: TLOGFONT;
  Cap: String;
  AX, AY: Integer;
  OldFont: HFont;
begin
  If Degree=0 Then Inherited Paint
  else
    begin
      if not Transparent then
        begin
          Canvas.Brush.Color := Color;
          Canvas.Brush.Style := bsSolid;
          Canvas.Fillrect(Rect(0,0,Width,Height));
        end;
      Canvas.Font.Assign(Font);
      GetObject(Canvas.Font.Handle,SizeOf(LogRec),Addr(LogRec));
      LogRec.lfEscapement := FDegree;
      OldFont := SelectObject(Canvas.Handle,CreateFontIndirect(LogRec));
      Cap:=Caption;
      CalcRotatedPosition(Canvas.TextWidth(Cap),Canvas.TextHeight(Cap),Degree,AX,AY,Width,Height,Alignment);
      ExtTextOut(Canvas.Handle, AX, AY, 0, NIL, @Cap[1], Length(Cap), nil);
      DeleteObject(SelectObject(Canvas.Handle,OldFont));
      PaintCorners;
    end;
  DrawFrame(Self,Canvas,GetClientRect);
end;

{--------------------------------------------------------------------------------}

procedure TQRRotatedExpr.Print(OfsX, OfsY: Integer);
var
  aValue : TQREvResult;
  LogRec: TLOGFONT;
  Cap: String;
  AX, AY: Integer;
  R: TRect;
  OldFont: HFont;
begin
  If Degree=0 Then
    begin
      Inherited Print(OfsX, OfsY);
      exit;
    end;
  With ParentReport.QRPrinter Do
    begin
      aValue := {Evaluator.Value;}Value;
      case aValue.Kind of
        resInt : Caption := FormatFloat(Mask, aValue.IntResult*1.0);
        resString : Caption := aValue.strResult;
        resDouble : Caption := FormatFloat(Mask,aValue.DblResult);
        resBool : if aValue.booResult then Caption := 'True' else Caption := 'False'; {<-- do not resource }
        resError : Caption := Expression;
      end;
      if not Transparent then
        begin
          Canvas.Brush.Color := Color;
          Canvas.Brush.Style := bsSolid;
          Canvas.Fillrect(Rect(XPos(OfsX + Size.Left),YPos(OfsY + Size.Top),
                               XPos(OfsX + Size.Left+Size.Width),YPos(OfsY + Size.Top+Size.Height)));
        end;
      Canvas.Font.Assign(Font);
      GetObject(Canvas.Font.Handle,SizeOf(LogRec),Addr(LogRec));
      LogRec.lfEscapement := FDegree;
      OldFont := SelectObject(Canvas.Handle,CreateFontIndirect(LogRec));
      Cap:=Caption;
      CalcRotatedPosition(Canvas.TextWidth(Cap),Canvas.TextHeight(Cap),Degree,AX,AY,Width,Height,Alignment);
      R:= Rect( XPos(Size.Left), YPos(Size.Top),
                XPos(Size.Left+Size.Width),
                YPos(Size.Top+Size.Height));
      ExtTextOut(Canvas.Handle, XPos(OfsX + Size.Left + AX/XFactor), YPos(OfsY + Size.Top + AY/YFactor),
                 ETO_CLIPPED, @R, @Cap[1], Length(Cap), nil);
      DeleteObject(SelectObject(Canvas.Handle,OldFont));

      if ParentReport.FinalPass and Frame.AnyFrame then
        PrintFrame(Self,ParentReport.QRPrinter.Canvas,
                   Rect(ParentReport.QRPrinter.XPos(OfsX + Size.Left),
                        ParentReport.QRPrinter.YPos(OfsY + Size.Top),
                        ParentReport.QRPrinter.XPos(OfsX + Size.Left + Size.Width),
                        ParentReport.QRPrinter.YPos(OfsY + Size.Top + Size.Height)),
                   ParentReport.QRPrinter.XFactor,
                   ParentReport.QRPrinter.YFactor);
      if ResetAfterPrint then Reset;
    end;
end;

{--------------------------------------------------------------------------------}

procedure TQRRotatedExpr.SetDegree(Value: Word);
begin
  If Value<>FDegree Then
    begin
      If Value<>0 Then AutoSize:=False;
      While Value>3600 Do Value:=Value-3600;
      FDegree:=Value;
      Invalidate;
    end;
end;

{--------------------------------------------------------------------------------}
{--------------------------------------------------------------------------------}

procedure Register;
begin
  RegisterComponents('QReport', [ TQRRotatedLabel, TQRRotatedExpr ]);
end;

end.
